/*
  
   Simple Reed Solomon Benchmark Program for Small CPU

   (C) Tak.Sugawara www.sugawara-systems.com
           
       

	Features : 5 Interleaves, max 3 bytes correctable per each interleave.( Maximum of 15 bytes  burst error to be corrected.)
		   1 Seed Byte per interleave. (Prevent miss-correction due to synchronization Error)
		   3 CRC Bytes for entire data.(Currently Disabled)
		   Hardaware Oriented Algorithm applied.			 

	Compile  Method: avr-gcc -g -Wall -O2 -mmcu=atmega103 -Tdata=0x801100 this_file
		   (Verilog `Define Statements should be changed per each compilation
		    Verilog Test Bench refers to absolute address in printxx functions.
                    See generated map file.)	  
		
     Decode Algorithm:

       x_equ.c  sigma3    -((-S3 + S1*y + S2*z)/S0)
                            S3 + S1*sigma2 + S2*sigma1
                 sigma3  =  ---------------------------
                                   S0

       y_equ.c  sigma2  (-(S2*S4) + S1*S5 + (S2*S3 - S1*S4)*z)/(-Power(S2,2) + S1*S3)
                           S2*S4  + S1*S5 + (S2*S3 + S1*S4)*sigma1
                  sigma2 = ----------------------------------------
                           S2*S2  + S1*S3

       z_equ.c  sigma1  is as follows.

   sigma1=  -(((-(S1*S2) + S0*S3)*(S1*S3 - S0*S4) -
       (-Power(S1,2) + S0*S2)*(S2*S3 - S0*S5))/
     (Power(-(S1*S2) + S0*S3,2) -
       (-Power(S1,2) + S0*S2)*(-Power(S2,2) + S0*S4)))        //

              (S1*S2 + S0*S3)*(S1*S3 + S0*S4)+ (S1*S1 + S0*S2)*(S2*S3 + S0*S5)
    sigma1 =  ----------------------------------------------------------------
              (S1*S2 + S0*S3)*(S1*S2 + S0*S3)+ (S1*S1 + S0*S2)*(S2*S2 + S0*S4)

          Let :
                A= (S1*S2 + S0*S3)
                B= (S1*S1 + S0*S2)
                        A*(S1*S3 + S0*S4)+ B*(S2*S3 + S0*S5)
              sigma1  = ------------------------------------
                        A*(S1*S2 + S0*S3)+ B*(S2*S2 + S0*S4)



   e3= (a*b*S0 - a*S1 - b*S1 + S2)/((a - c)*(b - c))
         i*j*S0+S1*(i+j)+S2
     =  --------------------------------------
          (i+k)*(j+k)
   e2= -((c*e3 - a*(e3 - S0) - S1)/(-a + b))
         k*e3+i*(e3+S0)+S1
     = --------------------
         i+j

   e1= -e2 - e3 + S0
     =e2+e3+S0

*/
#define ROM_TABLE

#define MAX_X   8
#define Degree 7
#define No_Error 0
#define Single_Error 1
#define Double_Error 2
#define Uncorrectable 3
#define Tripple_Error 4
#define C2_unco        0xffff


#define Bit_Width  8
#define LFSR    255     /* 2** 8 -1 */

#define NUMBER 18       /* Bytes per Interleave <=255 */
#define DATA_SIZE 47   // orig DATA_SIZE   EQUATION DATA_SIZE+CRC_SIZE= (NUMBER-1)*MAX_INTL-DEGREE*MAX_INTL

#define MAX_INTL 5
#define CRC_BYTE 3
#define SEED_BYTE 5
#define POLYNOMIAL      0x006928cdL


typedef unsigned char uchar;
typedef unsigned int  uint;
typedef unsigned long ulong;

//Globals
	unsigned char array[1000];//Internal Debug Use for myself
	unsigned char print_port;// String OUT port for verilog
	unsigned char print_char_port;// char OUT port for verilog
	unsigned int  print_int_port;//  int(16bit) OUT port for verilog
	unsigned long print_long_port;// long(32bit) OUT port for verilog
	


	unsigned char g1,g2,g3,g4;





/* Encode */
struct Mat_B {
 unsigned char Data_x[Bit_Width];
     } ;

struct Mat_B B0;// ={0,0,0,0,0,0,0,0};


 const unsigned char  e_temp0_uchar[]={0x1d ,0x3a ,0x69 ,0xce ,0x81 ,0x3 ,0x7 ,0xe };   
 const unsigned char  e_temp1_uchar[]={0x84 ,0x8 ,0x95 ,0xae ,0xd8 ,0xb0 ,0x61 ,0xc2 }; 
 const unsigned char  e_temp2_uchar[]={0xa1 ,0x43 ,0x26 ,0xed ,0x7a ,0xf4 ,0xe8 ,0xd0 };
 const unsigned char  e_temp3_uchar[]={0x2a ,0x54 ,0x83 ,0x2c ,0x72 ,0xe5 ,0xca ,0x95 };
 const unsigned char  e_temp4_uchar[]={0x52 ,0xa5 ,0x18 ,0x63 ,0x95 ,0x2a ,0x54 ,0xa9 };
 const unsigned char  e_temp5_uchar[]={0xfc ,0xf9 ,0xe ,0xe1 ,0x3f ,0x7f ,0xff ,0xfe }; 
 const unsigned char  e_temp6_uchar[]={0xbd ,0x7b ,0x4b ,0x2b ,0xeb ,0xd7 ,0xaf ,0x5e };
 const unsigned char  d_temp1_uchar[]={0x80 ,0x1 ,0x82 ,0x84 ,0x88 ,0x10 ,0x20 ,0x40 }; 
 const unsigned char  d_temp2_uchar[]={0x40 ,0x80 ,0x41 ,0xc2 ,0xc4 ,0x88 ,0x10 ,0x20 };
 const unsigned char  d_temp3_uchar[]={0x20 ,0x40 ,0xa0 ,0x61 ,0xe2 ,0xc4 ,0x88 ,0x10 };
 const unsigned char  d_temp4_uchar[]={0x10 ,0x20 ,0x50 ,0xb0 ,0x71 ,0xe2 ,0xc4 ,0x88 };
 const unsigned char  d_temp5_uchar[]={0x88 ,0x10 ,0xa8 ,0xd8 ,0x38 ,0x71 ,0xe2 ,0xc4 };
 const unsigned char  d_temp6_uchar[]={0xc4 ,0x88 ,0xd4 ,0x6c ,0x1c ,0x38 ,0x71 ,0xe2 };



const unsigned char glog[]={0x0,0x0,0x1,0x19,0x2,0x32,0x1a,0xc6,0x3,0xdf,0x33,0xee,0x1b,0x68,0xc7,0x4b
,0x4,0x64,0xe0,0xe,0x34,0x8d,0xef,0x81,0x1c,0xc1,0x69,0xf8,0xc8,0x8,0x4c,0x71
,0x5,0x8a,0x65,0x2f,0xe1,0x24,0xf,0x21,0x35,0x93,0x8e,0xda,0xf0,0x12,0x82,0x45
,0x1d,0xb5,0xc2,0x7d,0x6a,0x27,0xf9,0xb9,0xc9,0x9a,0x9,0x78,0x4d,0xe4,0x72,0xa6
,0x6,0xbf,0x8b,0x62,0x66,0xdd,0x30,0xfd,0xe2,0x98,0x25,0xb3,0x10,0x91,0x22,0x88
,0x36,0xd0,0x94,0xce,0x8f,0x96,0xdb,0xbd,0xf1,0xd2,0x13,0x5c,0x83,0x38,0x46,0x40
,0x1e,0x42,0xb6,0xa3,0xc3,0x48,0x7e,0x6e,0x6b,0x3a,0x28,0x54,0xfa,0x85,0xba,0x3d
,0xca,0x5e,0x9b,0x9f,0xa,0x15,0x79,0x2b,0x4e,0xd4,0xe5,0xac,0x73,0xf3,0xa7,0x57
,0x7,0x70,0xc0,0xf7,0x8c,0x80,0x63,0xd,0x67,0x4a,0xde,0xed,0x31,0xc5,0xfe,0x18
,0xe3,0xa5,0x99,0x77,0x26,0xb8,0xb4,0x7c,0x11,0x44,0x92,0xd9,0x23,0x20,0x89,0x2e
,0x37,0x3f,0xd1,0x5b,0x95,0xbc,0xcf,0xcd,0x90,0x87,0x97,0xb2,0xdc,0xfc,0xbe,0x61
,0xf2,0x56,0xd3,0xab,0x14,0x2a,0x5d,0x9e,0x84,0x3c,0x39,0x53,0x47,0x6d,0x41,0xa2
,0x1f,0x2d,0x43,0xd8,0xb7,0x7b,0xa4,0x76,0xc4,0x17,0x49,0xec,0x7f,0xc,0x6f,0xf6
,0x6c,0xa1,0x3b,0x52,0x29,0x9d,0x55,0xaa,0xfb,0x60,0x86,0xb1,0xbb,0xcc,0x3e,0x5a
,0xcb,0x59,0x5f,0xb0,0x9c,0xa9,0xa0,0x51,0xb,0xf5,0x16,0xeb,0x7a,0x75,0x2c,0xd7
,0x4f,0xae,0xd5,0xe9,0xe6,0xe7,0xad,0xe8,0x74,0xd6,0xf4,0xea,0xa8,0x50,0x58,0xaf
};
const unsigned char ganlog[]={0x1,0x2,0x4,0x8,0x10,0x20,0x40,0x80,0x1d,0x3a,0x74,0xe8,0xcd,0x87,0x13,0x26
,0x4c,0x98,0x2d,0x5a,0xb4,0x75,0xea,0xc9,0x8f,0x3,0x6,0xc,0x18,0x30,0x60,0xc0
,0x9d,0x27,0x4e,0x9c,0x25,0x4a,0x94,0x35,0x6a,0xd4,0xb5,0x77,0xee,0xc1,0x9f,0x23
,0x46,0x8c,0x5,0xa,0x14,0x28,0x50,0xa0,0x5d,0xba,0x69,0xd2,0xb9,0x6f,0xde,0xa1
,0x5f,0xbe,0x61,0xc2,0x99,0x2f,0x5e,0xbc,0x65,0xca,0x89,0xf,0x1e,0x3c,0x78,0xf0
,0xfd,0xe7,0xd3,0xbb,0x6b,0xd6,0xb1,0x7f,0xfe,0xe1,0xdf,0xa3,0x5b,0xb6,0x71,0xe2
,0xd9,0xaf,0x43,0x86,0x11,0x22,0x44,0x88,0xd,0x1a,0x34,0x68,0xd0,0xbd,0x67,0xce
,0x81,0x1f,0x3e,0x7c,0xf8,0xed,0xc7,0x93,0x3b,0x76,0xec,0xc5,0x97,0x33,0x66,0xcc
,0x85,0x17,0x2e,0x5c,0xb8,0x6d,0xda,0xa9,0x4f,0x9e,0x21,0x42,0x84,0x15,0x2a,0x54
,0xa8,0x4d,0x9a,0x29,0x52,0xa4,0x55,0xaa,0x49,0x92,0x39,0x72,0xe4,0xd5,0xb7,0x73
,0xe6,0xd1,0xbf,0x63,0xc6,0x91,0x3f,0x7e,0xfc,0xe5,0xd7,0xb3,0x7b,0xf6,0xf1,0xff
,0xe3,0xdb,0xab,0x4b,0x96,0x31,0x62,0xc4,0x95,0x37,0x6e,0xdc,0xa5,0x57,0xae,0x41
,0x82,0x19,0x32,0x64,0xc8,0x8d,0x7,0xe,0x1c,0x38,0x70,0xe0,0xdd,0xa7,0x53,0xa6
,0x51,0xa2,0x59,0xb2,0x79,0xf2,0xf9,0xef,0xc3,0x9b,0x2b,0x56,0xac,0x45,0x8a,0x9
,0x12,0x24,0x48,0x90,0x3d,0x7a,0xf4,0xf5,0xf7,0xf3,0xfb,0xeb,0xcb,0x8b,0xb,0x16
,0x2c,0x58,0xb0,0x7d,0xfa,0xe9,0xcf,0x83,0x1b,0x36,0x6c,0xd8,0xad,0x47,0x8e,0x1
};
const unsigned char square[]={0x0,0x1,0x4,0x5,0x10,0x11,0x14,0x15,0x40,0x41,0x44,0x45,0x50,0x51,0x54,0x55
,0x1d,0x1c,0x19,0x18,0xd,0xc,0x9,0x8,0x5d,0x5c,0x59,0x58,0x4d,0x4c,0x49,0x48
,0x74,0x75,0x70,0x71,0x64,0x65,0x60,0x61,0x34,0x35,0x30,0x31,0x24,0x25,0x20,0x21
,0x69,0x68,0x6d,0x6c,0x79,0x78,0x7d,0x7c,0x29,0x28,0x2d,0x2c,0x39,0x38,0x3d,0x3c
,0xcd,0xcc,0xc9,0xc8,0xdd,0xdc,0xd9,0xd8,0x8d,0x8c,0x89,0x88,0x9d,0x9c,0x99,0x98
,0xd0,0xd1,0xd4,0xd5,0xc0,0xc1,0xc4,0xc5,0x90,0x91,0x94,0x95,0x80,0x81,0x84,0x85
,0xb9,0xb8,0xbd,0xbc,0xa9,0xa8,0xad,0xac,0xf9,0xf8,0xfd,0xfc,0xe9,0xe8,0xed,0xec
,0xa4,0xa5,0xa0,0xa1,0xb4,0xb5,0xb0,0xb1,0xe4,0xe5,0xe0,0xe1,0xf4,0xf5,0xf0,0xf1
,0x13,0x12,0x17,0x16,0x3,0x2,0x7,0x6,0x53,0x52,0x57,0x56,0x43,0x42,0x47,0x46
,0xe,0xf,0xa,0xb,0x1e,0x1f,0x1a,0x1b,0x4e,0x4f,0x4a,0x4b,0x5e,0x5f,0x5a,0x5b
,0x67,0x66,0x63,0x62,0x77,0x76,0x73,0x72,0x27,0x26,0x23,0x22,0x37,0x36,0x33,0x32
,0x7a,0x7b,0x7e,0x7f,0x6a,0x6b,0x6e,0x6f,0x3a,0x3b,0x3e,0x3f,0x2a,0x2b,0x2e,0x2f
,0xde,0xdf,0xda,0xdb,0xce,0xcf,0xca,0xcb,0x9e,0x9f,0x9a,0x9b,0x8e,0x8f,0x8a,0x8b
,0xc3,0xc2,0xc7,0xc6,0xd3,0xd2,0xd7,0xd6,0x83,0x82,0x87,0x86,0x93,0x92,0x97,0x96
,0xaa,0xab,0xae,0xaf,0xba,0xbb,0xbe,0xbf,0xea,0xeb,0xee,0xef,0xfa,0xfb,0xfe,0xff
,0xb7,0xb6,0xb3,0xb2,0xa7,0xa6,0xa3,0xa2,0xf7,0xf6,0xf3,0xf2,0xe7,0xe6,0xe3,0xe2
};
const unsigned char gsol[]={0x0,0xd6,0xe8,0x3e,0xea,0x3c,0x2,0xd4,0x2c,0xfa,0xc4,0x12,0xc6,0x10,0x2e,0xf8
,0xee,0x38,0x6,0xd0,0x4,0xd2,0xec,0x3a,0xc2,0x14,0x2a,0xfc,0x28,0xfe,0xc0,0x16
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x24,0xf2,0xcc,0x1a,0xce,0x18,0x26,0xf0,0x8,0xde,0xe0,0x36,0xe2,0x34,0xa,0xdc
,0xca,0x1c,0x22,0xf4,0x20,0xf6,0xc8,0x1e,0xe6,0x30,0xe,0xd8,0xc,0xda,0xe4,0x32
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x50,0x86,0xb8,0x6e,0xba,0x6c,0x52,0x84,0x7c,0xaa,0x94,0x42,0x96,0x40,0x7e,0xa8
,0xbe,0x68,0x56,0x80,0x54,0x82,0xbc,0x6a,0x92,0x44,0x7a,0xac,0x78,0xae,0x90,0x46
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x74,0xa2,0x9c,0x4a,0x9e,0x48,0x76,0xa0,0x58,0x8e,0xb0,0x66,0xb2,0x64,0x5a,0x8c
,0x9a,0x4c,0x72,0xa4,0x70,0xa6,0x98,0x4e,0xb6,0x60,0x5e,0x88,0x5c,0x8a,0xb4,0x62
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
};
const unsigned char cube[]={0x0,0xd7,0x0,0x0,0x0,0x0,0x0,0xcb,0xb3,0x0,0xdc,0x0,0x71,0x0,0x0,0x67
,0x0,0x0,0x0,0x0,0x0,0xb8,0x0,0xf2,0x0,0x0,0xa7,0x0,0x0,0x0,0x0,0x0
,0x0,0xc3,0x0,0x0,0xe9,0xcd,0xff,0xe8,0x0,0xda,0x0,0x0,0xfd,0xe3,0x0,0xd0
,0x0,0x0,0x0,0x0,0x0,0xc4,0xfc,0x0,0xc2,0x0,0xfe,0x6a,0x0,0xd1,0xc5,0x0
,0x7f,0x0,0x0,0x0,0xdd,0x0,0x6e,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0xa5,0x0,0x0,0x0,0x0,0xac,0xca,0xf3,0x0,0x72,0x0,0x0,0x0,0x0,0x0,0x0
,0xe2,0xea,0x0,0x0,0xeb,0xbd,0xcc,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xf7,0x0
,0x0,0x0,0x0,0x0,0x0,0xdb,0x0,0x0,0xce,0x0,0x0,0x0,0x0,0xe7,0x0,0x0
,0x0,0x0,0xfb,0x0,0x0,0x0,0xe0,0x0,0x0,0x0,0xfa,0xf0,0x0,0x0,0x0,0xb6
,0x0,0xa0,0x0,0x0,0x0,0x0,0xf4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xef,0x0,0x0,0x0,0xe5,0x0,0x0
,0x0,0x0,0x0,0xba,0x0,0x0,0xf8,0x0,0xf9,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0xd5,0xf5,0xd4,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xf1,0xec,0xd3
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0xed,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0,0x0
};

const unsigned char e_0[]={0x0,0x75,0xea,0x9f,0xc9,0xbc,0x23,0x56,0x8f,0xfa,0x65,0x10,0x46,0x33,0xac,0xd9
,0x3,0x76,0xe9,0x9c,0xca,0xbf,0x20,0x55,0x8c,0xf9,0x66,0x13,0x45,0x30,0xaf,0xda
,0x6,0x73,0xec,0x99,0xcf,0xba,0x25,0x50,0x89,0xfc,0x63,0x16,0x40,0x35,0xaa,0xdf
,0x5,0x70,0xef,0x9a,0xcc,0xb9,0x26,0x53,0x8a,0xff,0x60,0x15,0x43,0x36,0xa9,0xdc
,0xc,0x79,0xe6,0x93,0xc5,0xb0,0x2f,0x5a,0x83,0xf6,0x69,0x1c,0x4a,0x3f,0xa0,0xd5
,0xf,0x7a,0xe5,0x90,0xc6,0xb3,0x2c,0x59,0x80,0xf5,0x6a,0x1f,0x49,0x3c,0xa3,0xd6
,0xa,0x7f,0xe0,0x95,0xc3,0xb6,0x29,0x5c,0x85,0xf0,0x6f,0x1a,0x4c,0x39,0xa6,0xd3
,0x9,0x7c,0xe3,0x96,0xc0,0xb5,0x2a,0x5f,0x86,0xf3,0x6c,0x19,0x4f,0x3a,0xa5,0xd0
,0x18,0x6d,0xf2,0x87,0xd1,0xa4,0x3b,0x4e,0x97,0xe2,0x7d,0x8,0x5e,0x2b,0xb4,0xc1
,0x1b,0x6e,0xf1,0x84,0xd2,0xa7,0x38,0x4d,0x94,0xe1,0x7e,0xb,0x5d,0x28,0xb7,0xc2
,0x1e,0x6b,0xf4,0x81,0xd7,0xa2,0x3d,0x48,0x91,0xe4,0x7b,0xe,0x58,0x2d,0xb2,0xc7
,0x1d,0x68,0xf7,0x82,0xd4,0xa1,0x3e,0x4b,0x92,0xe7,0x78,0xd,0x5b,0x2e,0xb1,0xc4
,0x14,0x61,0xfe,0x8b,0xdd,0xa8,0x37,0x42,0x9b,0xee,0x71,0x4,0x52,0x27,0xb8,0xcd
,0x17,0x62,0xfd,0x88,0xde,0xab,0x34,0x41,0x98,0xed,0x72,0x7,0x51,0x24,0xbb,0xce
,0x12,0x67,0xf8,0x8d,0xdb,0xae,0x31,0x44,0x9d,0xe8,0x77,0x2,0x54,0x21,0xbe,0xcb
,0x11,0x64,0xfb,0x8e,0xd8,0xad,0x32,0x47,0x9e,0xeb,0x74,0x1,0x57,0x22,0xbd,0xc8
};
const unsigned char e_1[]={0x0,0x44,0x88,0xcc,0xd,0x49,0x85,0xc1,0x1a,0x5e,0x92,0xd6,0x17,0x53,0x9f,0xdb
,0x34,0x70,0xbc,0xf8,0x39,0x7d,0xb1,0xf5,0x2e,0x6a,0xa6,0xe2,0x23,0x67,0xab,0xef
,0x68,0x2c,0xe0,0xa4,0x65,0x21,0xed,0xa9,0x72,0x36,0xfa,0xbe,0x7f,0x3b,0xf7,0xb3
,0x5c,0x18,0xd4,0x90,0x51,0x15,0xd9,0x9d,0x46,0x2,0xce,0x8a,0x4b,0xf,0xc3,0x87
,0xd0,0x94,0x58,0x1c,0xdd,0x99,0x55,0x11,0xca,0x8e,0x42,0x6,0xc7,0x83,0x4f,0xb
,0xe4,0xa0,0x6c,0x28,0xe9,0xad,0x61,0x25,0xfe,0xba,0x76,0x32,0xf3,0xb7,0x7b,0x3f
,0xb8,0xfc,0x30,0x74,0xb5,0xf1,0x3d,0x79,0xa2,0xe6,0x2a,0x6e,0xaf,0xeb,0x27,0x63
,0x8c,0xc8,0x4,0x40,0x81,0xc5,0x9,0x4d,0x96,0xd2,0x1e,0x5a,0x9b,0xdf,0x13,0x57
,0xbd,0xf9,0x35,0x71,0xb0,0xf4,0x38,0x7c,0xa7,0xe3,0x2f,0x6b,0xaa,0xee,0x22,0x66
,0x89,0xcd,0x1,0x45,0x84,0xc0,0xc,0x48,0x93,0xd7,0x1b,0x5f,0x9e,0xda,0x16,0x52
,0xd5,0x91,0x5d,0x19,0xd8,0x9c,0x50,0x14,0xcf,0x8b,0x47,0x3,0xc2,0x86,0x4a,0xe
,0xe1,0xa5,0x69,0x2d,0xec,0xa8,0x64,0x20,0xfb,0xbf,0x73,0x37,0xf6,0xb2,0x7e,0x3a
,0x6d,0x29,0xe5,0xa1,0x60,0x24,0xe8,0xac,0x77,0x33,0xff,0xbb,0x7a,0x3e,0xf2,0xb6
,0x59,0x1d,0xd1,0x95,0x54,0x10,0xdc,0x98,0x43,0x7,0xcb,0x8f,0x4e,0xa,0xc6,0x82
,0x5,0x41,0x8d,0xc9,0x8,0x4c,0x80,0xc4,0x1f,0x5b,0x97,0xd3,0x12,0x56,0x9a,0xde
,0x31,0x75,0xb9,0xfd,0x3c,0x78,0xb4,0xf0,0x2b,0x6f,0xa3,0xe7,0x26,0x62,0xae,0xea
};
const unsigned char e_2[]={0x0,0xb,0x16,0x1d,0x2c,0x27,0x3a,0x31,0x58,0x53,0x4e,0x45,0x74,0x7f,0x62,0x69
,0xb0,0xbb,0xa6,0xad,0x9c,0x97,0x8a,0x81,0xe8,0xe3,0xfe,0xf5,0xc4,0xcf,0xd2,0xd9
,0x7d,0x76,0x6b,0x60,0x51,0x5a,0x47,0x4c,0x25,0x2e,0x33,0x38,0x9,0x2,0x1f,0x14
,0xcd,0xc6,0xdb,0xd0,0xe1,0xea,0xf7,0xfc,0x95,0x9e,0x83,0x88,0xb9,0xb2,0xaf,0xa4
,0xfa,0xf1,0xec,0xe7,0xd6,0xdd,0xc0,0xcb,0xa2,0xa9,0xb4,0xbf,0x8e,0x85,0x98,0x93
,0x4a,0x41,0x5c,0x57,0x66,0x6d,0x70,0x7b,0x12,0x19,0x4,0xf,0x3e,0x35,0x28,0x23
,0x87,0x8c,0x91,0x9a,0xab,0xa0,0xbd,0xb6,0xdf,0xd4,0xc9,0xc2,0xf3,0xf8,0xe5,0xee
,0x37,0x3c,0x21,0x2a,0x1b,0x10,0xd,0x6,0x6f,0x64,0x79,0x72,0x43,0x48,0x55,0x5e
,0xe9,0xe2,0xff,0xf4,0xc5,0xce,0xd3,0xd8,0xb1,0xba,0xa7,0xac,0x9d,0x96,0x8b,0x80
,0x59,0x52,0x4f,0x44,0x75,0x7e,0x63,0x68,0x1,0xa,0x17,0x1c,0x2d,0x26,0x3b,0x30
,0x94,0x9f,0x82,0x89,0xb8,0xb3,0xae,0xa5,0xcc,0xc7,0xda,0xd1,0xe0,0xeb,0xf6,0xfd
,0x24,0x2f,0x32,0x39,0x8,0x3,0x1e,0x15,0x7c,0x77,0x6a,0x61,0x50,0x5b,0x46,0x4d
,0x13,0x18,0x5,0xe,0x3f,0x34,0x29,0x22,0x4b,0x40,0x5d,0x56,0x67,0x6c,0x71,0x7a
,0xa3,0xa8,0xb5,0xbe,0x8f,0x84,0x99,0x92,0xfb,0xf0,0xed,0xe6,0xd7,0xdc,0xc1,0xca
,0x6e,0x65,0x78,0x73,0x42,0x49,0x54,0x5f,0x36,0x3d,0x20,0x2b,0x1a,0x11,0xc,0x7
,0xde,0xd5,0xc8,0xc3,0xf2,0xf9,0xe4,0xef,0x86,0x8d,0x90,0x9b,0xaa,0xa1,0xbc,0xb7
};
const unsigned char e_3[]={0x0,0xa4,0x55,0xf1,0xaa,0xe,0xff,0x5b,0x49,0xed,0x1c,0xb8,0xe3,0x47,0xb6,0x12
,0x92,0x36,0xc7,0x63,0x38,0x9c,0x6d,0xc9,0xdb,0x7f,0x8e,0x2a,0x71,0xd5,0x24,0x80
,0x39,0x9d,0x6c,0xc8,0x93,0x37,0xc6,0x62,0x70,0xd4,0x25,0x81,0xda,0x7e,0x8f,0x2b
,0xab,0xf,0xfe,0x5a,0x1,0xa5,0x54,0xf0,0xe2,0x46,0xb7,0x13,0x48,0xec,0x1d,0xb9
,0x72,0xd6,0x27,0x83,0xd8,0x7c,0x8d,0x29,0x3b,0x9f,0x6e,0xca,0x91,0x35,0xc4,0x60
,0xe0,0x44,0xb5,0x11,0x4a,0xee,0x1f,0xbb,0xa9,0xd,0xfc,0x58,0x3,0xa7,0x56,0xf2
,0x4b,0xef,0x1e,0xba,0xe1,0x45,0xb4,0x10,0x2,0xa6,0x57,0xf3,0xa8,0xc,0xfd,0x59
,0xd9,0x7d,0x8c,0x28,0x73,0xd7,0x26,0x82,0x90,0x34,0xc5,0x61,0x3a,0x9e,0x6f,0xcb
,0xe4,0x40,0xb1,0x15,0x4e,0xea,0x1b,0xbf,0xad,0x9,0xf8,0x5c,0x7,0xa3,0x52,0xf6
,0x76,0xd2,0x23,0x87,0xdc,0x78,0x89,0x2d,0x3f,0x9b,0x6a,0xce,0x95,0x31,0xc0,0x64
,0xdd,0x79,0x88,0x2c,0x77,0xd3,0x22,0x86,0x94,0x30,0xc1,0x65,0x3e,0x9a,0x6b,0xcf
,0x4f,0xeb,0x1a,0xbe,0xe5,0x41,0xb0,0x14,0x6,0xa2,0x53,0xf7,0xac,0x8,0xf9,0x5d
,0x96,0x32,0xc3,0x67,0x3c,0x98,0x69,0xcd,0xdf,0x7b,0x8a,0x2e,0x75,0xd1,0x20,0x84
,0x4,0xa0,0x51,0xf5,0xae,0xa,0xfb,0x5f,0x4d,0xe9,0x18,0xbc,0xe7,0x43,0xb2,0x16
,0xaf,0xb,0xfa,0x5e,0x5,0xa1,0x50,0xf4,0xe6,0x42,0xb3,0x17,0x4c,0xe8,0x19,0xbd
,0x3d,0x99,0x68,0xcc,0x97,0x33,0xc2,0x66,0x74,0xd0,0x21,0x85,0xde,0x7a,0x8b,0x2f
};
const unsigned char e_4[]={0x0,0x9a,0x29,0xb3,0x52,0xc8,0x7b,0xe1,0xa4,0x3e,0x8d,0x17,0xf6,0x6c,0xdf,0x45
,0x55,0xcf,0x7c,0xe6,0x7,0x9d,0x2e,0xb4,0xf1,0x6b,0xd8,0x42,0xa3,0x39,0x8a,0x10
,0xaa,0x30,0x83,0x19,0xf8,0x62,0xd1,0x4b,0xe,0x94,0x27,0xbd,0x5c,0xc6,0x75,0xef
,0xff,0x65,0xd6,0x4c,0xad,0x37,0x84,0x1e,0x5b,0xc1,0x72,0xe8,0x9,0x93,0x20,0xba
,0x49,0xd3,0x60,0xfa,0x1b,0x81,0x32,0xa8,0xed,0x77,0xc4,0x5e,0xbf,0x25,0x96,0xc
,0x1c,0x86,0x35,0xaf,0x4e,0xd4,0x67,0xfd,0xb8,0x22,0x91,0xb,0xea,0x70,0xc3,0x59
,0xe3,0x79,0xca,0x50,0xb1,0x2b,0x98,0x2,0x47,0xdd,0x6e,0xf4,0x15,0x8f,0x3c,0xa6
,0xb6,0x2c,0x9f,0x5,0xe4,0x7e,0xcd,0x57,0x12,0x88,0x3b,0xa1,0x40,0xda,0x69,0xf3
,0x92,0x8,0xbb,0x21,0xc0,0x5a,0xe9,0x73,0x36,0xac,0x1f,0x85,0x64,0xfe,0x4d,0xd7
,0xc7,0x5d,0xee,0x74,0x95,0xf,0xbc,0x26,0x63,0xf9,0x4a,0xd0,0x31,0xab,0x18,0x82
,0x38,0xa2,0x11,0x8b,0x6a,0xf0,0x43,0xd9,0x9c,0x6,0xb5,0x2f,0xce,0x54,0xe7,0x7d
,0x6d,0xf7,0x44,0xde,0x3f,0xa5,0x16,0x8c,0xc9,0x53,0xe0,0x7a,0x9b,0x1,0xb2,0x28
,0xdb,0x41,0xf2,0x68,0x89,0x13,0xa0,0x3a,0x7f,0xe5,0x56,0xcc,0x2d,0xb7,0x4,0x9e
,0x8e,0x14,0xa7,0x3d,0xdc,0x46,0xf5,0x6f,0x2a,0xb0,0x3,0x99,0x78,0xe2,0x51,0xcb
,0x71,0xeb,0x58,0xc2,0x23,0xb9,0xa,0x90,0xd5,0x4f,0xfc,0x66,0x87,0x1d,0xae,0x34
,0x24,0xbe,0xd,0x97,0x76,0xec,0x5f,0xc5,0x80,0x1a,0xa9,0x33,0xd2,0x48,0xfb,0x61
};
const unsigned char e_5[]={0x0,0x7a,0xf4,0x8e,0xf5,0x8f,0x1,0x7b,0xf7,0x8d,0x3,0x79,0x2,0x78,0xf6,0x8c
,0xf3,0x89,0x7,0x7d,0x6,0x7c,0xf2,0x88,0x4,0x7e,0xf0,0x8a,0xf1,0x8b,0x5,0x7f
,0xfb,0x81,0xf,0x75,0xe,0x74,0xfa,0x80,0xc,0x76,0xf8,0x82,0xf9,0x83,0xd,0x77
,0x8,0x72,0xfc,0x86,0xfd,0x87,0x9,0x73,0xff,0x85,0xb,0x71,0xa,0x70,0xfe,0x84
,0xeb,0x91,0x1f,0x65,0x1e,0x64,0xea,0x90,0x1c,0x66,0xe8,0x92,0xe9,0x93,0x1d,0x67
,0x18,0x62,0xec,0x96,0xed,0x97,0x19,0x63,0xef,0x95,0x1b,0x61,0x1a,0x60,0xee,0x94
,0x10,0x6a,0xe4,0x9e,0xe5,0x9f,0x11,0x6b,0xe7,0x9d,0x13,0x69,0x12,0x68,0xe6,0x9c
,0xe3,0x99,0x17,0x6d,0x16,0x6c,0xe2,0x98,0x14,0x6e,0xe0,0x9a,0xe1,0x9b,0x15,0x6f
,0xcb,0xb1,0x3f,0x45,0x3e,0x44,0xca,0xb0,0x3c,0x46,0xc8,0xb2,0xc9,0xb3,0x3d,0x47
,0x38,0x42,0xcc,0xb6,0xcd,0xb7,0x39,0x43,0xcf,0xb5,0x3b,0x41,0x3a,0x40,0xce,0xb4
,0x30,0x4a,0xc4,0xbe,0xc5,0xbf,0x31,0x4b,0xc7,0xbd,0x33,0x49,0x32,0x48,0xc6,0xbc
,0xc3,0xb9,0x37,0x4d,0x36,0x4c,0xc2,0xb8,0x34,0x4e,0xc0,0xba,0xc1,0xbb,0x35,0x4f
,0x20,0x5a,0xd4,0xae,0xd5,0xaf,0x21,0x5b,0xd7,0xad,0x23,0x59,0x22,0x58,0xd6,0xac
,0xd3,0xa9,0x27,0x5d,0x26,0x5c,0xd2,0xa8,0x24,0x5e,0xd0,0xaa,0xd1,0xab,0x25,0x5f
,0xdb,0xa1,0x2f,0x55,0x2e,0x54,0xda,0xa0,0x2c,0x56,0xd8,0xa2,0xd9,0xa3,0x2d,0x57
,0x28,0x52,0xdc,0xa6,0xdd,0xa7,0x29,0x53,0xdf,0xa5,0x2b,0x51,0x2a,0x50,0xde,0xa4
};
const unsigned char e_6[]={0x0,0x7f,0xfe,0x81,0xe1,0x9e,0x1f,0x60,0xdf,0xa0,0x21,0x5e,0x3e,0x41,0xc0,0xbf
,0xa3,0xdc,0x5d,0x22,0x42,0x3d,0xbc,0xc3,0x7c,0x3,0x82,0xfd,0x9d,0xe2,0x63,0x1c
,0x5b,0x24,0xa5,0xda,0xba,0xc5,0x44,0x3b,0x84,0xfb,0x7a,0x5,0x65,0x1a,0x9b,0xe4
,0xf8,0x87,0x6,0x79,0x19,0x66,0xe7,0x98,0x27,0x58,0xd9,0xa6,0xc6,0xb9,0x38,0x47
,0xb6,0xc9,0x48,0x37,0x57,0x28,0xa9,0xd6,0x69,0x16,0x97,0xe8,0x88,0xf7,0x76,0x9
,0x15,0x6a,0xeb,0x94,0xf4,0x8b,0xa,0x75,0xca,0xb5,0x34,0x4b,0x2b,0x54,0xd5,0xaa
,0xed,0x92,0x13,0x6c,0xc,0x73,0xf2,0x8d,0x32,0x4d,0xcc,0xb3,0xd3,0xac,0x2d,0x52
,0x4e,0x31,0xb0,0xcf,0xaf,0xd0,0x51,0x2e,0x91,0xee,0x6f,0x10,0x70,0xf,0x8e,0xf1
,0x71,0xe,0x8f,0xf0,0x90,0xef,0x6e,0x11,0xae,0xd1,0x50,0x2f,0x4f,0x30,0xb1,0xce
,0xd2,0xad,0x2c,0x53,0x33,0x4c,0xcd,0xb2,0xd,0x72,0xf3,0x8c,0xec,0x93,0x12,0x6d
,0x2a,0x55,0xd4,0xab,0xcb,0xb4,0x35,0x4a,0xf5,0x8a,0xb,0x74,0x14,0x6b,0xea,0x95
,0x89,0xf6,0x77,0x8,0x68,0x17,0x96,0xe9,0x56,0x29,0xa8,0xd7,0xb7,0xc8,0x49,0x36
,0xc7,0xb8,0x39,0x46,0x26,0x59,0xd8,0xa7,0x18,0x67,0xe6,0x99,0xf9,0x86,0x7,0x78
,0x64,0x1b,0x9a,0xe5,0x85,0xfa,0x7b,0x4,0xbb,0xc4,0x45,0x3a,0x5a,0x25,0xa4,0xdb
,0x9c,0xe3,0x62,0x1d,0x7d,0x2,0x83,0xfc,0x43,0x3c,0xbd,0xc2,0xa2,0xdd,0x5c,0x23
,0x3f,0x40,0xc1,0xbe,0xde,0xa1,0x20,0x5f,0xe0,0x9f,0x1e,0x61,0x1,0x7e,0xff,0x80
};
const unsigned char d_1[]={0x0,0x2,0x4,0x6,0x8,0xa,0xc,0xe,0x10,0x12,0x14,0x16,0x18,0x1a,0x1c,0x1e
,0x20,0x22,0x24,0x26,0x28,0x2a,0x2c,0x2e,0x30,0x32,0x34,0x36,0x38,0x3a,0x3c,0x3e
,0x40,0x42,0x44,0x46,0x48,0x4a,0x4c,0x4e,0x50,0x52,0x54,0x56,0x58,0x5a,0x5c,0x5e
,0x60,0x62,0x64,0x66,0x68,0x6a,0x6c,0x6e,0x70,0x72,0x74,0x76,0x78,0x7a,0x7c,0x7e
,0x80,0x82,0x84,0x86,0x88,0x8a,0x8c,0x8e,0x90,0x92,0x94,0x96,0x98,0x9a,0x9c,0x9e
,0xa0,0xa2,0xa4,0xa6,0xa8,0xaa,0xac,0xae,0xb0,0xb2,0xb4,0xb6,0xb8,0xba,0xbc,0xbe
,0xc0,0xc2,0xc4,0xc6,0xc8,0xca,0xcc,0xce,0xd0,0xd2,0xd4,0xd6,0xd8,0xda,0xdc,0xde
,0xe0,0xe2,0xe4,0xe6,0xe8,0xea,0xec,0xee,0xf0,0xf2,0xf4,0xf6,0xf8,0xfa,0xfc,0xfe
,0x1d,0x1f,0x19,0x1b,0x15,0x17,0x11,0x13,0xd,0xf,0x9,0xb,0x5,0x7,0x1,0x3
,0x3d,0x3f,0x39,0x3b,0x35,0x37,0x31,0x33,0x2d,0x2f,0x29,0x2b,0x25,0x27,0x21,0x23
,0x5d,0x5f,0x59,0x5b,0x55,0x57,0x51,0x53,0x4d,0x4f,0x49,0x4b,0x45,0x47,0x41,0x43
,0x7d,0x7f,0x79,0x7b,0x75,0x77,0x71,0x73,0x6d,0x6f,0x69,0x6b,0x65,0x67,0x61,0x63
,0x9d,0x9f,0x99,0x9b,0x95,0x97,0x91,0x93,0x8d,0x8f,0x89,0x8b,0x85,0x87,0x81,0x83
,0xbd,0xbf,0xb9,0xbb,0xb5,0xb7,0xb1,0xb3,0xad,0xaf,0xa9,0xab,0xa5,0xa7,0xa1,0xa3
,0xdd,0xdf,0xd9,0xdb,0xd5,0xd7,0xd1,0xd3,0xcd,0xcf,0xc9,0xcb,0xc5,0xc7,0xc1,0xc3
,0xfd,0xff,0xf9,0xfb,0xf5,0xf7,0xf1,0xf3,0xed,0xef,0xe9,0xeb,0xe5,0xe7,0xe1,0xe3
};
const unsigned char d_2[]={0x0,0x4,0x8,0xc,0x10,0x14,0x18,0x1c,0x20,0x24,0x28,0x2c,0x30,0x34,0x38,0x3c
,0x40,0x44,0x48,0x4c,0x50,0x54,0x58,0x5c,0x60,0x64,0x68,0x6c,0x70,0x74,0x78,0x7c
,0x80,0x84,0x88,0x8c,0x90,0x94,0x98,0x9c,0xa0,0xa4,0xa8,0xac,0xb0,0xb4,0xb8,0xbc
,0xc0,0xc4,0xc8,0xcc,0xd0,0xd4,0xd8,0xdc,0xe0,0xe4,0xe8,0xec,0xf0,0xf4,0xf8,0xfc
,0x1d,0x19,0x15,0x11,0xd,0x9,0x5,0x1,0x3d,0x39,0x35,0x31,0x2d,0x29,0x25,0x21
,0x5d,0x59,0x55,0x51,0x4d,0x49,0x45,0x41,0x7d,0x79,0x75,0x71,0x6d,0x69,0x65,0x61
,0x9d,0x99,0x95,0x91,0x8d,0x89,0x85,0x81,0xbd,0xb9,0xb5,0xb1,0xad,0xa9,0xa5,0xa1
,0xdd,0xd9,0xd5,0xd1,0xcd,0xc9,0xc5,0xc1,0xfd,0xf9,0xf5,0xf1,0xed,0xe9,0xe5,0xe1
,0x3a,0x3e,0x32,0x36,0x2a,0x2e,0x22,0x26,0x1a,0x1e,0x12,0x16,0xa,0xe,0x2,0x6
,0x7a,0x7e,0x72,0x76,0x6a,0x6e,0x62,0x66,0x5a,0x5e,0x52,0x56,0x4a,0x4e,0x42,0x46
,0xba,0xbe,0xb2,0xb6,0xaa,0xae,0xa2,0xa6,0x9a,0x9e,0x92,0x96,0x8a,0x8e,0x82,0x86
,0xfa,0xfe,0xf2,0xf6,0xea,0xee,0xe2,0xe6,0xda,0xde,0xd2,0xd6,0xca,0xce,0xc2,0xc6
,0x27,0x23,0x2f,0x2b,0x37,0x33,0x3f,0x3b,0x7,0x3,0xf,0xb,0x17,0x13,0x1f,0x1b
,0x67,0x63,0x6f,0x6b,0x77,0x73,0x7f,0x7b,0x47,0x43,0x4f,0x4b,0x57,0x53,0x5f,0x5b
,0xa7,0xa3,0xaf,0xab,0xb7,0xb3,0xbf,0xbb,0x87,0x83,0x8f,0x8b,0x97,0x93,0x9f,0x9b
,0xe7,0xe3,0xef,0xeb,0xf7,0xf3,0xff,0xfb,0xc7,0xc3,0xcf,0xcb,0xd7,0xd3,0xdf,0xdb
};
const unsigned char d_3[]={0x0,0x8,0x10,0x18,0x20,0x28,0x30,0x38,0x40,0x48,0x50,0x58,0x60,0x68,0x70,0x78
,0x80,0x88,0x90,0x98,0xa0,0xa8,0xb0,0xb8,0xc0,0xc8,0xd0,0xd8,0xe0,0xe8,0xf0,0xf8
,0x1d,0x15,0xd,0x5,0x3d,0x35,0x2d,0x25,0x5d,0x55,0x4d,0x45,0x7d,0x75,0x6d,0x65
,0x9d,0x95,0x8d,0x85,0xbd,0xb5,0xad,0xa5,0xdd,0xd5,0xcd,0xc5,0xfd,0xf5,0xed,0xe5
,0x3a,0x32,0x2a,0x22,0x1a,0x12,0xa,0x2,0x7a,0x72,0x6a,0x62,0x5a,0x52,0x4a,0x42
,0xba,0xb2,0xaa,0xa2,0x9a,0x92,0x8a,0x82,0xfa,0xf2,0xea,0xe2,0xda,0xd2,0xca,0xc2
,0x27,0x2f,0x37,0x3f,0x7,0xf,0x17,0x1f,0x67,0x6f,0x77,0x7f,0x47,0x4f,0x57,0x5f
,0xa7,0xaf,0xb7,0xbf,0x87,0x8f,0x97,0x9f,0xe7,0xef,0xf7,0xff,0xc7,0xcf,0xd7,0xdf
,0x74,0x7c,0x64,0x6c,0x54,0x5c,0x44,0x4c,0x34,0x3c,0x24,0x2c,0x14,0x1c,0x4,0xc
,0xf4,0xfc,0xe4,0xec,0xd4,0xdc,0xc4,0xcc,0xb4,0xbc,0xa4,0xac,0x94,0x9c,0x84,0x8c
,0x69,0x61,0x79,0x71,0x49,0x41,0x59,0x51,0x29,0x21,0x39,0x31,0x9,0x1,0x19,0x11
,0xe9,0xe1,0xf9,0xf1,0xc9,0xc1,0xd9,0xd1,0xa9,0xa1,0xb9,0xb1,0x89,0x81,0x99,0x91
,0x4e,0x46,0x5e,0x56,0x6e,0x66,0x7e,0x76,0xe,0x6,0x1e,0x16,0x2e,0x26,0x3e,0x36
,0xce,0xc6,0xde,0xd6,0xee,0xe6,0xfe,0xf6,0x8e,0x86,0x9e,0x96,0xae,0xa6,0xbe,0xb6
,0x53,0x5b,0x43,0x4b,0x73,0x7b,0x63,0x6b,0x13,0x1b,0x3,0xb,0x33,0x3b,0x23,0x2b
,0xd3,0xdb,0xc3,0xcb,0xf3,0xfb,0xe3,0xeb,0x93,0x9b,0x83,0x8b,0xb3,0xbb,0xa3,0xab
};
const unsigned char d_4[]={0x0,0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x80,0x90,0xa0,0xb0,0xc0,0xd0,0xe0,0xf0
,0x1d,0xd,0x3d,0x2d,0x5d,0x4d,0x7d,0x6d,0x9d,0x8d,0xbd,0xad,0xdd,0xcd,0xfd,0xed
,0x3a,0x2a,0x1a,0xa,0x7a,0x6a,0x5a,0x4a,0xba,0xaa,0x9a,0x8a,0xfa,0xea,0xda,0xca
,0x27,0x37,0x7,0x17,0x67,0x77,0x47,0x57,0xa7,0xb7,0x87,0x97,0xe7,0xf7,0xc7,0xd7
,0x74,0x64,0x54,0x44,0x34,0x24,0x14,0x4,0xf4,0xe4,0xd4,0xc4,0xb4,0xa4,0x94,0x84
,0x69,0x79,0x49,0x59,0x29,0x39,0x9,0x19,0xe9,0xf9,0xc9,0xd9,0xa9,0xb9,0x89,0x99
,0x4e,0x5e,0x6e,0x7e,0xe,0x1e,0x2e,0x3e,0xce,0xde,0xee,0xfe,0x8e,0x9e,0xae,0xbe
,0x53,0x43,0x73,0x63,0x13,0x3,0x33,0x23,0xd3,0xc3,0xf3,0xe3,0x93,0x83,0xb3,0xa3
,0xe8,0xf8,0xc8,0xd8,0xa8,0xb8,0x88,0x98,0x68,0x78,0x48,0x58,0x28,0x38,0x8,0x18
,0xf5,0xe5,0xd5,0xc5,0xb5,0xa5,0x95,0x85,0x75,0x65,0x55,0x45,0x35,0x25,0x15,0x5
,0xd2,0xc2,0xf2,0xe2,0x92,0x82,0xb2,0xa2,0x52,0x42,0x72,0x62,0x12,0x2,0x32,0x22
,0xcf,0xdf,0xef,0xff,0x8f,0x9f,0xaf,0xbf,0x4f,0x5f,0x6f,0x7f,0xf,0x1f,0x2f,0x3f
,0x9c,0x8c,0xbc,0xac,0xdc,0xcc,0xfc,0xec,0x1c,0xc,0x3c,0x2c,0x5c,0x4c,0x7c,0x6c
,0x81,0x91,0xa1,0xb1,0xc1,0xd1,0xe1,0xf1,0x1,0x11,0x21,0x31,0x41,0x51,0x61,0x71
,0xa6,0xb6,0x86,0x96,0xe6,0xf6,0xc6,0xd6,0x26,0x36,0x6,0x16,0x66,0x76,0x46,0x56
,0xbb,0xab,0x9b,0x8b,0xfb,0xeb,0xdb,0xcb,0x3b,0x2b,0x1b,0xb,0x7b,0x6b,0x5b,0x4b
};
const unsigned char d_5[]={0x0,0x20,0x40,0x60,0x80,0xa0,0xc0,0xe0,0x1d,0x3d,0x5d,0x7d,0x9d,0xbd,0xdd,0xfd
,0x3a,0x1a,0x7a,0x5a,0xba,0x9a,0xfa,0xda,0x27,0x7,0x67,0x47,0xa7,0x87,0xe7,0xc7
,0x74,0x54,0x34,0x14,0xf4,0xd4,0xb4,0x94,0x69,0x49,0x29,0x9,0xe9,0xc9,0xa9,0x89
,0x4e,0x6e,0xe,0x2e,0xce,0xee,0x8e,0xae,0x53,0x73,0x13,0x33,0xd3,0xf3,0x93,0xb3
,0xe8,0xc8,0xa8,0x88,0x68,0x48,0x28,0x8,0xf5,0xd5,0xb5,0x95,0x75,0x55,0x35,0x15
,0xd2,0xf2,0x92,0xb2,0x52,0x72,0x12,0x32,0xcf,0xef,0x8f,0xaf,0x4f,0x6f,0xf,0x2f
,0x9c,0xbc,0xdc,0xfc,0x1c,0x3c,0x5c,0x7c,0x81,0xa1,0xc1,0xe1,0x1,0x21,0x41,0x61
,0xa6,0x86,0xe6,0xc6,0x26,0x6,0x66,0x46,0xbb,0x9b,0xfb,0xdb,0x3b,0x1b,0x7b,0x5b
,0xcd,0xed,0x8d,0xad,0x4d,0x6d,0xd,0x2d,0xd0,0xf0,0x90,0xb0,0x50,0x70,0x10,0x30
,0xf7,0xd7,0xb7,0x97,0x77,0x57,0x37,0x17,0xea,0xca,0xaa,0x8a,0x6a,0x4a,0x2a,0xa
,0xb9,0x99,0xf9,0xd9,0x39,0x19,0x79,0x59,0xa4,0x84,0xe4,0xc4,0x24,0x4,0x64,0x44
,0x83,0xa3,0xc3,0xe3,0x3,0x23,0x43,0x63,0x9e,0xbe,0xde,0xfe,0x1e,0x3e,0x5e,0x7e
,0x25,0x5,0x65,0x45,0xa5,0x85,0xe5,0xc5,0x38,0x18,0x78,0x58,0xb8,0x98,0xf8,0xd8
,0x1f,0x3f,0x5f,0x7f,0x9f,0xbf,0xdf,0xff,0x2,0x22,0x42,0x62,0x82,0xa2,0xc2,0xe2
,0x51,0x71,0x11,0x31,0xd1,0xf1,0x91,0xb1,0x4c,0x6c,0xc,0x2c,0xcc,0xec,0x8c,0xac
,0x6b,0x4b,0x2b,0xb,0xeb,0xcb,0xab,0x8b,0x76,0x56,0x36,0x16,0xf6,0xd6,0xb6,0x96
};
const unsigned char d_6[]={0x0,0x40,0x80,0xc0,0x1d,0x5d,0x9d,0xdd,0x3a,0x7a,0xba,0xfa,0x27,0x67,0xa7,0xe7
,0x74,0x34,0xf4,0xb4,0x69,0x29,0xe9,0xa9,0x4e,0xe,0xce,0x8e,0x53,0x13,0xd3,0x93
,0xe8,0xa8,0x68,0x28,0xf5,0xb5,0x75,0x35,0xd2,0x92,0x52,0x12,0xcf,0x8f,0x4f,0xf
,0x9c,0xdc,0x1c,0x5c,0x81,0xc1,0x1,0x41,0xa6,0xe6,0x26,0x66,0xbb,0xfb,0x3b,0x7b
,0xcd,0x8d,0x4d,0xd,0xd0,0x90,0x50,0x10,0xf7,0xb7,0x77,0x37,0xea,0xaa,0x6a,0x2a
,0xb9,0xf9,0x39,0x79,0xa4,0xe4,0x24,0x64,0x83,0xc3,0x3,0x43,0x9e,0xde,0x1e,0x5e
,0x25,0x65,0xa5,0xe5,0x38,0x78,0xb8,0xf8,0x1f,0x5f,0x9f,0xdf,0x2,0x42,0x82,0xc2
,0x51,0x11,0xd1,0x91,0x4c,0xc,0xcc,0x8c,0x6b,0x2b,0xeb,0xab,0x76,0x36,0xf6,0xb6
,0x87,0xc7,0x7,0x47,0x9a,0xda,0x1a,0x5a,0xbd,0xfd,0x3d,0x7d,0xa0,0xe0,0x20,0x60
,0xf3,0xb3,0x73,0x33,0xee,0xae,0x6e,0x2e,0xc9,0x89,0x49,0x9,0xd4,0x94,0x54,0x14
,0x6f,0x2f,0xef,0xaf,0x72,0x32,0xf2,0xb2,0x55,0x15,0xd5,0x95,0x48,0x8,0xc8,0x88
,0x1b,0x5b,0x9b,0xdb,0x6,0x46,0x86,0xc6,0x21,0x61,0xa1,0xe1,0x3c,0x7c,0xbc,0xfc
,0x4a,0xa,0xca,0x8a,0x57,0x17,0xd7,0x97,0x70,0x30,0xf0,0xb0,0x6d,0x2d,0xed,0xad
,0x3e,0x7e,0xbe,0xfe,0x23,0x63,0xa3,0xe3,0x4,0x44,0x84,0xc4,0x19,0x59,0x99,0xd9
,0xa2,0xe2,0x22,0x62,0xbf,0xff,0x3f,0x7f,0x98,0xd8,0x18,0x58,0x85,0xc5,0x5,0x45
,0xd6,0x96,0x56,0x16,0xcb,0x8b,0x4b,0xb,0xec,0xac,0x6c,0x2c,0xf1,0xb1,0x71,0x31
};



unsigned char transB() ;
struct Syndrome { unsigned char S0,S1,S2,S3,S4,S5,S6;} Syndrome;
struct C1 { int error_type;
            int number_of_errors;
                  int position0;
                  int position1;
                  int position2;
                  unsigned char error_value0;
                  unsigned char error_value1;
                  unsigned char error_value2;
                } C1[MAX_INTL];
struct C1_Error { int error_type;
                  int position0;
                  int position1;
                  unsigned char false_byte0;
                  unsigned char corrected_byte0;
                  unsigned char false_byte1;
                  unsigned char corrected_byte1;
                } C1_Error;


struct error_pos{
                int     sector;
                int     byte_position;
                int     n_of_byte;
                unsigned char     initial_value;
                unsigned char     false_value[256];
                unsigned char     correct_value;
                };


	int sub(int);




unsigned  char WD[1000];
unsigned  char disk_data[1000];
unsigned  char save_data[1000];
struct Syn { unsigned char S0,S1,S2,S3,S4,S5,S6;} Syn[MAX_INTL];
struct Seed { unsigned char Byte[5];} Seed;
unsigned long CRC=0x00000000L;
unsigned char *crc_pointer;

unsigned char gmul(unsigned char u,unsigned char v);
unsigned char constant_mul(unsigned char* a_ptr,unsigned char b);
void Error_Sim0();
void three_error_correction(int, int);


//Verilog Specific Routines
void print(unsigned char* ptr)//Verilog Test Bench Use 
{
	while (*ptr) {
		print_port=*(ptr++);
	}
	print_port=0x00;//Write Done
}

void print_char(unsigned char val)//Little Endian write out 16bit number 
{
	print_char_port=(unsigned char)val ;
	

}
void print_int(int val)//Little Endian write out 16bit number 
{	
	print_int_port=val ;

	

}

void print_long(unsigned long val)//Little Endian write out 32bit number 
{
	print_long_port=val;


}

//End of Verilog Specific Routines

unsigned char constant_mul(unsigned char* a_ptr,unsigned char b)
{
   int i,j,k;
   
	unsigned char temp[Bit_Width];
	unsigned char result_temp[Bit_Width];

	unsigned char result;
        for (i=0;i<Bit_Width ;i++ ) {
                temp[i]=a_ptr[i] & b ;
		result_temp[i]=0;
		for (j=0; j<Bit_Width;j++) {
			result_temp[i] ^=(temp[i] >> j )& 0x01; 
		}
         }
	result=0;
	for (i=0;i<Bit_Width ;i++ ) {
		result |=result_temp[i] << i ;
	}
       return(result);

}
void clear_syndrome()
{
        int j;

      for (j=0;j<MAX_INTL;j++) {
                Syn[j].S6=0;
                Syn[j].S5=0;
                Syn[j].S4=0;
                Syn[j].S3=0;
                Syn[j].S2=0;
                Syn[j].S1=0;
                Syn[j].S0=0;
      }
}
void buffer_compare()

{
    
  int flag=0;
	int i;
	for (i=0;i<DATA_SIZE;i++) {     
		  if ( save_data[i] !=disk_data[i]){
			  print("Compare Error Occurred.\n");
			  flag=1;
		  }
        }
	if (!flag)   print("Compare Done successfully\n");
   }

   
#ifndef ROM_TABLE
void make_endec_table()
{
int i;

      for (i=0;i<=LFSR;i++){
                e_0[i]= constant_mul(&e_temp0_uchar,i);//transB(mat_mulB( e_temp0 ,temp ))  ;
                e_1[i]= constant_mul(&e_temp1_uchar,i);//transB(mat_mulB( e_temp1 ,temp ))  ;
                e_2[i]= constant_mul(&e_temp2_uchar,i);//transB(mat_mulB( e_temp2 ,temp ))  ;
                e_3[i]= constant_mul(&e_temp3_uchar,i);//transB(mat_mulB( e_temp3 ,temp ))  ;
                e_4[i]= constant_mul(&e_temp4_uchar,i);//transB(mat_mulB( e_temp4 ,temp ))  ;
                e_5[i]= constant_mul(&e_temp5_uchar,i);//transB(mat_mulB( e_temp5 ,temp ))  ;
                e_6[i]= constant_mul(&e_temp6_uchar,i);//transB(mat_mulB( e_temp6 ,temp ))  ;
                d_1[i]= constant_mul(&d_temp1_uchar,i);//transB(mat_mulB( d_temp1 ,temp ))  ;
                d_2[i]= constant_mul(&d_temp2_uchar,i);//transB(mat_mulB( d_temp2 ,temp ))  ;
                d_3[i]= constant_mul(&d_temp3_uchar,i);//transB(mat_mulB( d_temp3 ,temp ))  ;
                d_4[i]= constant_mul(&d_temp4_uchar,i);//transB(mat_mulB( d_temp4 ,temp ))  ;
                d_5[i]= constant_mul(&d_temp5_uchar,i);//transB(mat_mulB( d_temp5 ,temp ))  ;
                d_6[i]= constant_mul(&d_temp6_uchar,i);//transB(mat_mulB( d_temp6 ,temp ))  ;
		print("Making Endec Matrix\n");

      }
}
#endif
void encode(int INTL,unsigned char write_data)
{
        unsigned char a;

                       a= Syn[INTL].S6^write_data ;
                       Syn[INTL].S6= e_6[a]^Syn[INTL].S5 ;
                       Syn[INTL].S5= e_5[a]^Syn[INTL].S4;
                       Syn[INTL].S4= e_4[a]^Syn[INTL].S3  ;
                       Syn[INTL].S3= e_3[a]^Syn[INTL].S2   ;
                       Syn[INTL].S2= e_2[a]^Syn[INTL].S1    ;
                       Syn[INTL].S1= e_1[a]^Syn[INTL].S0     ;
                       Syn[INTL].S0= e_0[a];

}


void decode(int INTL,unsigned char read_data)
{
                    Syn[INTL].S0=read_data^Syn[INTL].S0;
                    Syn[INTL].S1=read_data^d_1[Syn[INTL].S1];
                    Syn[INTL].S2=read_data^d_2[Syn[INTL].S2];
                    Syn[INTL].S3=read_data^d_3[Syn[INTL].S3];
                    Syn[INTL].S4=read_data^d_4[Syn[INTL].S4];
                    Syn[INTL].S5=read_data^d_5[Syn[INTL].S5];
                    Syn[INTL].S6=read_data^d_6[Syn[INTL].S6];
}
int solve(int i,int j)
{


    return( (gmul(j,j) & 0xff) ^ j== i       );




   }






//Jan.19.2004@SDCC Note:@(u0 & v0) + (u7 & v1)@ƂȂƌ듮BʂȂƃG[ɔY܂B
unsigned char gmul(unsigned char u,unsigned char v)

{
     unsigned char u0,u1,u2,u3,u4,u5,u6,u7;
     unsigned char  v0,v1,v2,v3,v4,v5,v6,v7;
     unsigned char uv0,uv1,uv2,uv3,uv4,uv5,uv6,uv7;
     unsigned char uv; 



     u0= u & 0x01    ;
     u1= (u>>1) & 0x01   ;
     u2= (u>>2) & 0x01  ;
     u3= (u>>3) & 0x01  ;
     u4= (u>>4) & 0x01  ;
     u5= (u>>5) & 0x01  ;
     u6= (u>>6) & 0x01  ;
     u7= (u>>7) & 0x01  ;

     v0= v & 0x01    ;
     v1= (v>>1) & 0x01   ;
     v2= (v>>2) & 0x01  ;
     v3= (v>>3) & 0x01  ;
     v4= (v>>4) & 0x01  ;
     v5= (v>>5) & 0x01  ;
     v6= (v>>6) & 0x01  ;
     v7= (v>>7) & 0x01  ;


uv0=( (u0&v0) + (u7&v1) + (u6&v2) + (u5&v3) + (u4&v4) + (u3&v5) + (u7&v5) + (u2&v6) + (u6&v6) +
             (u7&v6) + (u1&v7) + (u5&v7) + (u6&v7) + (u7&v7) ) %2 ;
	
uv1=((u1&v0) + (u0&v1) + (u7&v2) + (u6&v3) + (u5&v4) + (u4&v5) + (u3&v6) + (u7&v6) + (u2&v7) +
   (u6&v7) + (u7&v7))%2;

uv2=( (u2&v0) + (u1&v1) + (u7&v1) + (u0&v2) + (u6&v2) + (u5&v3) + (u7&v3) + (u4&v4) + (u6&v4) +
   (u3&v5) + (u5&v5) + (u7&v5) + (u2&v6) + (u4&v6) + (u6&v6) + (u7&v6) + (u1&v7) + (u3&v7) +
   (u5&v7) + (u6&v7))%2;
uv3=( (u3&v0) + (u2&v1) + (u7&v1) + (u1&v2) + (u6&v2) + (u7&v2) + (u0&v3) + (u5&v3) + (u6&v3) +
   (u4&v4) + (u5&v4) + (u7&v4) + (u3&v5) + (u4&v5) + (u6&v5) + (u7&v5) + (u2&v6) + (u3&v6) +
   (u5&v6) + (u6&v6) + (u1&v7) + (u2&v7) + (u4&v7) + (u5&v7))%2;
uv4=( (u4&v0) + (u3&v1) + (u7&v1) + (u2&v2) + (u6&v2) + (u7&v2) + (u1&v3) + (u5&v3) + (u6&v3) +
      (u7&v3) + (u0&v4) + (u4&v4) + (u5&v4) + (u6&v4) + (u3&v5) + (u4&v5) + (u5&v5) + (u2&v6) +
      (u3&v6) + (u4&v6) + (u1&v7) + (u2&v7) + (u3&v7) + (u7&v7) )%2;
uv5=( (u5&v0) + (u4&v1) + (u3&v2) + (u7&v2) + (u2&v3) + (u6&v3) + (u7&v3) + (u1&v4) + (u5&v4) +
      (u6&v4) + (u7&v4) + (u0&v5) + (u4&v5) + (u5&v5) + (u6&v5) + (u3&v6) + (u4&v6) + (u5&v6) +
      (u2&v7) + (u3&v7) + (u4&v7)    )%2;
uv6=( (u6&v0) + (u5&v1) + (u4&v2) + (u3&v3) + (u7&v3) + (u2&v4) + (u6&v4) + (u7&v4) + (u1&v5) +
      (u5&v5) + (u6&v5) + (u7&v5) + (u0&v6) + (u4&v6) + (u5&v6) + (u6&v6) + (u3&v7) + (u4&v7) +
      (u5&v7)    )%2;
uv7=( (u7&v0) + (u6&v1) + (u5&v2) + (u4&v3) + (u3&v4) + (u7&v4) + (u2&v5) + (u6&v5) + (u7&v5) +
      (u1&v6) + (u5&v6) + (u6&v6) + (u7&v6) + (u0&v7) + (u4&v7) + (u5&v7) + (u6&v7))%2;
      




uv= (uv7<<7) & 0x80  |
    (uv6<<6) & 0x40  |
    (uv5<<5) & 0x20  |
    (uv4<<4) & 0x10  |
    (uv3<<3) & 0x08  |
    (uv2<<2) & 0x04  |
    (uv1<<1) & 0x02  |
    (uv0   ) & 0x01  ;

   return(uv);
}
unsigned char alpha_mult(unsigned char i,unsigned char k)
{
         int n,ii,kk;
        unsigned char m;

        if (i==0 || k==0) {
           return(0);
        } else {
              ii=glog[i];
              kk=glog[k];
              n=ii+kk;
              m=n%LFSR;
              return(ganlog[m]);
        }
}





unsigned char alpha_div(unsigned char i,unsigned char k)      /* i/k */
{
         int n,ii,kk;
        unsigned char m;
/* This routine assumes k !=0
   Upper module must ensure k !=0  */
         if ( (i==0) || (k==0)) {
            return(0);
         } else {
	      
              ii=glog[i];
              kk=glog[k];
              n=ii-kk;
              if (n<=0) {
                 n=n+LFSR;
              } else {
              }
              m=n%LFSR;
	


              return(ganlog[m]);
         }

}
#ifndef ROM_TABLE
void make_log()
{

        int data[MAX_X];
        int old_data[MAX_X];
        int flag;
        int i,j,k;


        for (i=0;i<MAX_X ;i++ ) {
                data[i]=0;
                old_data[i]=0;
        }

                old_data[7]=0;
                flag=0;
        for (i=0;i<=LFSR ;i++ ) {
           if (flag==0) {
                data[0]=1;        /* 1*/
                flag=1;
           } else {
                data[0]=old_data[7];        /* 1*/
           }



                data[1]=old_data[0];
                data[2]=old_data[1]^ old_data[7] ;
                data[3]=old_data[2]^old_data[7];
                data[4]=old_data[3]^ old_data[7] ;
                data[5]=old_data[4];
                data[6]=old_data[5];
                data[7]=old_data[6];


                old_data[0]=data[0];
                old_data[1]=data[1];
                old_data[2]=data[2];
                old_data[3]=data[3];
                old_data[4]=data[4];
                old_data[5]=data[5];
                old_data[6]=data[6];
                old_data[7]=data[7];


                k=0;
                    k=128*data[7]+64*data[6]+32*data[5]+16*data[4]+8*data[3]+4*data[2]+2*data[1]+data[0];
                      glog[k]=i;
                      ganlog[i]=k;
		     


        }
   

/* Compensation */


                glog[1]=0;   /* 255-> 0 */


       
}
void make_gsol()
{
    int i,j,k,n;
    unsigned char   unchar0,unchar1,unchar2;

   for (j=0;j<=LFSR;j++ ) {
                square[j]= gmul(j,j);
   }	

    for (i=0;i<=LFSR ;i++ ) {
       gsol[i]=0;
       for (j=0;j<=LFSR;j++ ) {
                unchar0= square[j];//gmul(j,j);
                unchar1=(unsigned char) j;
                unchar2=(unsigned char) i;

                unchar0= unchar0 ^ unchar1 ^ unchar2;
                if ( unchar0==0 ) {

                 gsol[i]=unchar1;
		 break;
                } else {

                }
       }
       array[64]=gsol[i];	
         	
    }



}

void make_cube()
{
    int j;
    unsigned char   unchar0;

       for (j=0;j<=LFSR;j++ ) {
                cube[j]=0;
                unchar0= square[j];//gmul(j,j);
                unchar0= gmul(unchar0,j);
                cube[unchar0]=(unsigned char)j ;
		array[65]=j;	

       }

}
#endif
void init_glovals()
{
	int i;
	for (i=0; i< Bit_Width;i++) {
		B0.Data_x[i]=0;
	}

}


void main()
{
	
#ifndef ROM_TABLE
	make_log();
	make_gsol();
	make_cube()     ;
	make_endec_table();

#endif


// Test Start
        Error_Sim0()    ;
	print("$finish");
// Test Finished


	
}

int sub(int a)
{
		
	return a*2;

}
void func_crc(unsigned char  i )
{
   int j;
   unsigned  char ii;
   unsigned  char crc1;



    for (j=0;j<=7;j++) {
        ii= (i << j )     & 0x80;
        crc1= (CRC>>16)   & 0x80    ;


        if (ii ^ crc1  ) {
                   CRC=((CRC<<1) ^ POLYNOMIAL )& 0x00ffffffL   ;
        } else {
                   CRC=(CRC<<1) & 0x00ffffffL;
        }
    }
    /*
        printf("CRC=%8x\n",CRC);
        getchar();
    */
}
void single_error_correction( unsigned char i,int num,int INTL)

{
        C1[INTL].position0=num-1-i   ;
        C1[INTL].error_value0=       Syn[INTL].S0  ;

}



void double_error_correction( int num, int INTL)

{

         unsigned char s0s3,s0s2,s1s1,s1s3,s1s2,s2s2;
         unsigned char sigma1,sigma2,c,a,b,d,e,f;
         unsigned int x1,x2,i,j,error0,error1;
         unsigned char x1_temp,x2_temp;


        s0s3=alpha_mult(Syn[INTL].S0,Syn[INTL].S3);
        s0s2=alpha_mult(Syn[INTL].S0,Syn[INTL].S2);
        s1s1=alpha_mult(Syn[INTL].S1,Syn[INTL].S1);
        s1s3=alpha_mult(Syn[INTL].S1,Syn[INTL].S3);
        s1s2=alpha_mult(Syn[INTL].S1,Syn[INTL].S2);
        s2s2=alpha_mult(Syn[INTL].S2,Syn[INTL].S2);

                            a=s0s3^s1s2;
                            b=s2s2^s1s3;
                            c=s0s2^s1s1;

        if (a==0 ) {
              C1[INTL].error_type=Uncorrectable;
              return;
        } else {
        }
        if ( b==0 ) {
              C1[INTL].error_type=Uncorrectable;
              return;
        } else {
        }
        if ( c==0) {
              C1[INTL].error_type=Uncorrectable;
              return;
        } else {
        }


        sigma1=alpha_div(a,c);
        sigma2=alpha_div(b,c);



        a=alpha_mult(sigma2,Syn[INTL].S2)  ;
        b=alpha_mult(sigma1,Syn[INTL].S3)  ;
        c=alpha_mult(sigma2,Syn[INTL].S3)  ;
        d=alpha_mult(sigma1,Syn[INTL].S4)  ;
/*  added for c1c7 */

        e=alpha_mult(sigma2,Syn[INTL].S4)  ;
        f=alpha_mult(sigma1,Syn[INTL].S5)  ;


        if
                ((a ^ b) ==Syn[INTL].S4)

         {      
		 if ((c ^ d) !=Syn[INTL].S5) {
			
                  C1[INTL].error_type=Tripple_Error;
                  three_error_correction( num,INTL) ;
                  return;
                } else {
			
                   if ((e^f) == Syn[INTL].S6) {
					
                   } else {
			
                   C1[INTL].error_type=Tripple_Error;
                   three_error_correction( num,INTL) ;
                   return;
                   }
                }
        } else {	
			
                   C1[INTL].error_type=Tripple_Error;
                  three_error_correction( num,INTL) ;
              return;
        }


/* Double Tenatative  Valid  */
        c=alpha_div(sigma2,alpha_mult(sigma1,sigma1));
        x1=alpha_mult(sigma1,gsol[c]);
        x2=alpha_mult(sigma1,gsol[c] ^0x01);
        j=glog[x2];
        i=glog[x1];



        error0=alpha_div(alpha_mult(x2,Syn[INTL].S0)^Syn[INTL].S1,x1^x2);
        error1=error0^Syn[INTL].S0;

        C1[INTL].error_type=Double_Error;
        C1[INTL].position0=num-1-i;
        C1[INTL].error_value0=error0;
        C1[INTL].position1=num-1-j;
        C1[INTL].error_value1=error1;

}




void three_error_correction(int num,int INTL)
{

         unsigned char s0s3,s0s2,s1s1,s1s3,s1s2,s2s2;
         unsigned char s0s4,s0s5;
         unsigned char s1s4,s1s5,s2s3,s2s4;

         unsigned char sigma1_nu,sigma1_de;
         unsigned char sigma2_nu;
         unsigned char sigma3_nu;
         unsigned char sigma1,sigma2,c,a,b,d,e,f;
         unsigned char sigma3;
         unsigned int i,j,k,error1,error2,error3,error0;
         unsigned char x1_temp,x2_temp;
         unsigned char w,sig_diff;

         unsigned c1,c_numerator,c_denominator,u1,v1,t1,t2,t3,x1,x2,x3,K;
         int L1,L2,L3;
        struct Mat_B transform();




/* Solve Error position operator */
 /* sigma1 */
     /*
        sigma1

              (S1*S2 + S0*S3)*(S1*S3 + S0*S4)+ (S1*S1 + S0*S2)*(S2*S3 + S0*S5)
    sigma1 =  ----------------------------------------------------------------
              (S1*S2 + S0*S3)*(S1*S2 + S0*S3)+ (S1*S1 + S0*S2)*(S2*S2 + S0*S4)

          Let :
                a= (S1*S2 + S0*S3)
                c= (S1*S1 + S0*S2)

                        a*(S1*S3 + S0*S4)+ c*(S2*S3 + S0*S5)
              sigma1  = ------------------------------------
                        a*(S1*S2 + S0*S3)+ c*(S2*S2 + S0*S4)
      */
        s0s3=alpha_mult(Syn[INTL].S0,Syn[INTL].S3);
        s0s2=alpha_mult(Syn[INTL].S0,Syn[INTL].S2);
        s1s1=alpha_mult(Syn[INTL].S1,Syn[INTL].S1);
        s1s3=alpha_mult(Syn[INTL].S1,Syn[INTL].S3);
        s1s2=alpha_mult(Syn[INTL].S1,Syn[INTL].S2);
        s2s2=alpha_mult(Syn[INTL].S2,Syn[INTL].S2);



        s0s4=alpha_mult(Syn[INTL].S0,Syn[INTL].S4);
        s0s5=alpha_mult(Syn[INTL].S0,Syn[INTL].S5);
        s2s3=alpha_mult(Syn[INTL].S2,Syn[INTL].S3);



         a=s0s3^s1s2;
         b=s2s2^s1s3;
         c=s0s2^s1s1;

        sigma1_nu=alpha_mult(a,(s1s3 ^ s0s4)) ^ alpha_mult(c,(s2s3 ^ s0s5));
        sigma1_de=alpha_mult(a,a            ) ^ alpha_mult(c,(s2s2 ^ s0s4));

        /* Note div0 check to be necessary */
        sigma1=alpha_div(sigma1_nu,sigma1_de);


 /* sigma2 */
        /*
                           S2*S4  + S1*S5 + (S2*S3 + S1*S4)*sigma1
                  sigma2 = ----------------------------------------
                           S2*S2  + S1*S3
                           S2*S4  + S1*S5 + (S2*S3 + S1*S4)*sigma1
                         = --------------------------------------
                              b
        */

        s2s4=alpha_mult(Syn[INTL].S2,Syn[INTL].S4);
        s1s4=alpha_mult(Syn[INTL].S1,Syn[INTL].S4);
        s1s5=alpha_mult(Syn[INTL].S1,Syn[INTL].S5);


        sigma2_nu= s2s4^s1s5^ alpha_mult((s2s3^s1s4),sigma1);

        /* Note div0 check to be necessary */
        sigma2=alpha_div(sigma2_nu,b);



 /* sigma3 */
    /*
                            S3 + S1*sigma2 + S2*sigma1
                 sigma3  =  ---------------------------
                                   S0

    */


        sigma3_nu= Syn[INTL].S3^alpha_mult(Syn[INTL].S1,sigma2)^alpha_mult(Syn[INTL].S2,sigma1);

        /* Note div0 check to be necessary */
        sigma3=alpha_div(sigma3_nu,Syn[INTL].S0);

  
/* Solve e3
         i*j*S0+S1*(i+j)+S2
   e3 =  --------------------------------------
          (i+k)*(j+k)

    use ganlog[] for i,j,k;
   */

  


        K=alpha_mult(sigma1,sigma1) ^ sigma2;
        c_numerator=alpha_mult(alpha_mult(K,K),K);
        c1=alpha_mult(sigma1,sigma2)^sigma3 ;
        c_denominator=alpha_mult(c1,c1);
        c=alpha_div(c_numerator,c_denominator);
        v1=gsol[c];
        u1=alpha_mult(v1,c1);
        t1=cube[u1];
        x1=sigma1^t1^alpha_div(K,t1);
        L1=glog[x1];
        f=ganlog[85];
        t2=alpha_mult(t1,f);
        t3=alpha_mult(t2,f);
        x2=sigma1^t2^alpha_div(K,t2);
        x3=sigma1^t3^alpha_div(K,t3);
        L2=glog[x2];
        L3=glog[x3];
    




        i=x1;
        j=x2;
        k=x3;

        error3=   alpha_div(
                  alpha_mult(alpha_mult(i,j),Syn[INTL].S0)^
                  alpha_mult(i^j,Syn[INTL].S1)^
                  Syn[INTL].S2 ,
                  alpha_mult(i^k,j^k) ) ;


/* Solve e2
         k*e3+i*(e3+S0)+S1
   e2 = --------------------
         i+j
*/
        error2=   alpha_div(
                  alpha_mult(k,error3)^
                  alpha_mult(i,error3^Syn[INTL].S0)^
                  Syn[INTL].S1 ,
                  i^j ) ;




/* Solve e1
    e1=e2+e3+S0
*/
        error1= error2^error3^Syn[INTL].S0;

        C1[INTL].error_type=Tripple_Error;
        C1[INTL].position0=num-1-L1;
        C1[INTL].error_value0=error1;
        C1[INTL].position1=num-1-L2;
        C1[INTL].error_value1=error2;
        C1[INTL].position2=num-1-L3;
        C1[INTL].error_value2=error3;




        return;



}

void Syndrome_Analysis( int num, int degree, int INTL)

{

        unsigned char a;
        int error_type;

	


                 if (Syn[INTL].S0==0 &&
                     Syn[INTL].S1==0 &&
                     Syn[INTL].S2==0 &&
                     Syn[INTL].S3==0 &&
                     Syn[INTL].S4==0 &&
                     Syn[INTL].S5==0 &&
                     Syn[INTL].S6==0
                 ){
			

                          C1[INTL].error_type=No_Error;
                 } else {
                    if (Syn[INTL].S0==0 || Syn[INTL].S1==0) {
                          C1[INTL].error_type=Double_Error;
			  	
                  } else {

                      a=alpha_div(Syn[INTL].S1,Syn[INTL].S0);
                      if ( alpha_mult(a,Syn[INTL].S1) !=Syn[INTL].S2 ) {
                           C1[INTL].error_type=Double_Error;
			  	

                      }

                   else   if ( alpha_mult(a,Syn[INTL].S2) !=Syn[INTL].S3 ) {
                           C1[INTL].error_type=Double_Error;
						  	

                      }

                   else   if ( alpha_mult(a,Syn[INTL].S3) !=Syn[INTL].S4 ) {
                           C1[INTL].error_type=Double_Error;
							  	

                      }

                   else   if ( alpha_mult(a,Syn[INTL].S4) !=Syn[INTL].S5 ) {
                           C1[INTL].error_type=Double_Error;
							  	

                      } else {

                           C1[INTL].error_type=Single_Error;


                       }
                  }
                }

                switch (C1[INTL].error_type) {
                    case No_Error:
                       break;
                    case Single_Error:
                        single_error_correction( glog[a], num,INTL)     ;
                       break;
                    case Double_Error:
                        double_error_correction(  num,INTL)       ;
                       break;
                    default:
                       break;
                    } /* endswitch */
}




void Error_Sim0()
{
   int i;
   int INTL=0;
   int j;
   int disk_byte_ctr;
   unsigned char write_data,read_data;

	
	print("Start Simple Reed Solomon Test Bench on SH2 7047.");
	print(" (C) www.sugawara-systems.com\n");
	print(" Build Jan.31.2004 \n");
 	print("$time");//For Bench Mark

            Seed.Byte[0]=0;      /* Byte 0 is always 0. which is interleave 4 */
            Seed.Byte[1]=0;      /*                                         3 */
            Seed.Byte[2]=0;      /*                                         2 */
            Seed.Byte[3]=0;      /*                                         1 */
            Seed.Byte[4]=0;      /*                                         0 */

            for (i=0;i<DATA_SIZE;i++) {   /* Fill Data Buffer with incremental pattern.*/
                WD[i]=(unsigned char)i%256;
            }

            clear_syndrome();     /* Initialize ECC */
            CRC=0L;
            crc_pointer=&CRC;
            crc_pointer+=2;       /* Get Byte3 -24 bit*/

            disk_byte_ctr=0;

            for (i=0;i<MAX_INTL;i++) {          //Write Seed Bytes
                    write_data=Seed.Byte[i];
                    INTL=4-i%5;                 // Modulo Counter
                    //func_crc(write_data);
                    encode(INTL,write_data);
            }
	    print("Seed Bytes Out Done\n");
	    print("Original Data before Error error addition \n");
            for (i=0;i<DATA_SIZE+CRC_BYTE;i++) {      // Write (DATA+CRC) Bytes
                    INTL=4-i%5;                 // Modulo Counter
                    if (i<DATA_SIZE) {
                         write_data=WD[i];      // DATA Bytes
                        // func_crc(write_data);
                    } else write_data=*crc_pointer--;      // CRC Bytes
                    encode(INTL,write_data);
		    save_data[disk_byte_ctr]=write_data;	
                    disk_data[disk_byte_ctr++]=write_data;
		    print_char(write_data);

             }
	     print("\n");
	

            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S6;
            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S5;
            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S4;
            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S3;
            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S2;
            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S1;
            for (i=0;i<MAX_INTL;i++)   disk_data[disk_byte_ctr++]=Syn[4-i%5].S0;

        //	print("disk_byte_counter=");
	//	print_int(disk_byte_ctr);
	

        

            clear_syndrome();   // Initialize Syndrome Register
            CRC=0L;

            for (i=0;i<MAX_INTL;i++) {          //Read Seed Bytes
                    read_data=Seed.Byte[i];
                    INTL=4-i%5;                 // Modulo Counter
                    //func_crc(read_data)  ;
                    decode(4-i%5,read_data);
            }
   

     /* Make Error */

                 disk_data[1] ^=0xff;
		 disk_data[6] ^=0xaa;
		  disk_data[11] ^=0xcc;
		   disk_data[2] ^=0xbb;
		   disk_data[7] ^=0xdd;
		   disk_data[3] ^=0xee;

                 //disk_data[511+3] ^=0x01;
                 //disk_data[511+35] ^=0xff;



	    print("Decoding \n");

            for (i=0;i<(NUMBER-1)*MAX_INTL;i++) {
                read_data=disk_data[i];
                if (i<DATA_SIZE+CRC_BYTE) {  // func_crc(read_data)  ;
                             if (i>=DATA_SIZE) ;//printf("CRC READ=%2x\n",read_data);
                }
                decode(4-i%5,read_data);
		print_char(read_data);
            }
	    print("\n Syndrome Analysis begins..");
	    for (INTL=0;INTL<5;INTL++)  Syndrome_Analysis( NUMBER, Degree, INTL) ;
            print("C1 Correction Result\n");

        for (INTL=0;INTL<MAX_INTL;INTL++)   {
            print("Interleave= :  ");
	    print_int(INTL);
	    print("\t");
	   // print ("Error Type=");	
	   // print_int(C1[INTL].error_type);
	   // print ("\n"); 
	   
            switch (C1[INTL].error_type ) {
                case No_Error: print("No Error Detected.\n");
                               break;
                case Single_Error: print("Single Error Detected.\n");
				   
                               print("\t\t\tError Position=");
			       print_int(C1[INTL].position0);
			       print("\n\t\t\tError Value   =");
			       print_char(C1[INTL].error_value0);
			       print("\n");	
			       disk_data[ (C1[INTL].position0-1)*MAX_INTL+(4-INTL)] ^=C1[INTL].error_value0;//Correction	
	
                               break;
                case Double_Error: print("Double Error Detected.\n");
                               print("\t\t\tError Position=");
			       print_int(C1[INTL].position0);
			       print("\n\t\t\tError Value   =");
			       print_char(C1[INTL].error_value0);
			 
				
			       print("\n\t\t\tError Position=");
			       print_int(C1[INTL].position1);
			       print("\n\t\t\tError Value   =");
			       print_char(C1[INTL].error_value1);
			       print("\n");	
			      	 
			       disk_data[ (C1[INTL].position0-1)*MAX_INTL+(4-INTL)] ^=C1[INTL].error_value0;//Correction	
                               disk_data[ (C1[INTL].position1-1)*MAX_INTL+(4-INTL)] ^=C1[INTL].error_value1;//Correction	

                               break;
                case Tripple_Error: print("Tripple Error Detected.\n");
                              print("\t\t\tError Position=");
			       print_int(C1[INTL].position0);
			       print("\n\t\t\tError Value   =");
			       print_char(C1[INTL].error_value0);
			 
				
			       print("\n\t\t\tError Position=");
			       print_int(C1[INTL].position1);
			       print("\n\t\t\tError Value   =");
			       print_char(C1[INTL].error_value1);
			       
			       print("\n\t\t\tError Position=");
			       print_int(C1[INTL].position2);
			       print("\n\t\t\tError Value   =");
			       print_char(C1[INTL].error_value2);
			       print("\n");

		               disk_data[ (C1[INTL].position0-1)*MAX_INTL+(4-INTL)] ^=C1[INTL].error_value0;//Correction	
                               disk_data[ (C1[INTL].position1-1)*MAX_INTL+(4-INTL)] ^=C1[INTL].error_value1;//Correction	
                               disk_data[ (C1[INTL].position2-1)*MAX_INTL+(4-INTL)] ^=C1[INTL].error_value2;//Correction	

                               break;
		default:       print("Uncorrectable Error");
				break;
            }
         }

	print("After Correcting Errors\n");
	for (i=0;i<DATA_SIZE;i++) {     
                   print_char(disk_data[i]);
        }
	print("\n");
	buffer_compare();//Test Done.!
 	print("$time");//For Bench Mark
	print("$finish");//Exit Simulation

        
 }

/* Verilog Test Bench ,Insert sram write process with following verilog description
   `define address should be changed per compilation. See map file at compile time.
     
      `define Print_Port_Address 16'he7c9  //SH2
      `define Print_CAHR_Port_Address 16'hdfe8
      `define Print_INT_Port_Address 16'he7cc  //First ADDRESS
      `define Print_LONG_Port_Address 16'hdfec  //First ADDRESS

	task Cprint;// String OUT until the byte 00 or xx detected with least Byte first and justified.
		integer i;
		begin :Block
			i=0;
			while (1) begin
				if (char_buffer[i*8 +:8] ===8'h00 || char_buffer[i*8 +:8]===8'hxx) begin
						disable Block;
				end	
				$write("%c",char_buffer[i*8 +:8]);
				i=i+1;  
			end
		end
	endtask

   reg [0:639] char_buffer;
   integer  counter=0; 
   reg flag=0;   //Jan.31.2004 To address multi-cycle write for BYTE and WORD 
   
   always @ (posedge CLK ) begin
            if ((WE === 1'b1) && !flag) begin//Jan.31.2004       
           	   if (ADR[15:0]==`Print_Port_Address) begin
				flag=1;
				if (DATI[15: 8]===8'h00) begin
					char_buffer[counter*8 +:8]=DATI[15: 8];
					if (char_buffer[0  +:8*7]==="$finish") begin
							$stop;				
							
					end else if (char_buffer[0  +:8*5]==="$time") begin
							$display("Current Time on Simulation=%d",$time);				
							
					end else  Cprint;//$write("%s",char_buffer);

					for (counter=0; counter< 80; counter=counter+1) begin
						char_buffer[counter*8 +:8]=8'h00;
					end
					counter=0;
				
				end else begin
					char_buffer[counter*8 +:8]=DATI[15: 8];
					counter=counter+1;
				end
		   end
       	   else if (ADR[15:0]==`Print_CAHR_Port_Address ) begin
				flag=1;
				$write("%h ",DATI[7: 0]);
		   end else if (ADR[15:0]==`Print_INT_Port_Address) begin
				flag=1;
				#10 $write("%h ",DATI[15:0]);//Little Endian 
		   end else if (ADR[15:0]==`Print_LONG_Port_Address) begin
				#10 $write("%h ",DATI[31:0]);//Big Endian
		   end 
	end //if
	else if (!WE) flag=0;//Jan.31.2004
   end //always
   
*/
